@@ -791,8 +791,8 @@ autoar_extractor_signal_conflict (AutoarExtractor  *self,
     return AUTOAR_CONFLICT_SKIP;
 
   if (*new_file) {
-    g_autofree char *previous_path;
-    g_autofree char *new_path;
+    g_autofree char *previous_path = NULL;
+    g_autofree char *new_path = NULL;
 
     previous_path = g_file_get_path (file);
     new_path = g_file_get_path (*new_file);
@@ -857,6 +857,14 @@ autoar_extractor_get_common_prefix (GList *files,
   while (!g_file_has_parent (prefix, root)) {
     file = g_file_get_parent (prefix);
     g_object_unref (prefix);
+
+    /* This can happen if the archive contains the "/" path and the destination
+     * is "/" as well.
+     */
+    if (file == NULL) {
+      return NULL;
+    }
+
     prefix = file;
   }
 
@@ -878,8 +886,8 @@ autoar_extractor_do_sanitize_pathname (AutoarExtractor *self,
 {
   GFile *extracted_filename;
   gboolean valid_filename;
-  g_autofree char *sanitized_pathname;
-  g_autofree char *utf8_pathname;
+  g_autofree char *sanitized_pathname = NULL;
+  g_autofree char *utf8_pathname = NULL;
   GFile *destination;
 
   /* Use output_file when called from autoar_extractor_step_scan_toplevel(). */
@@ -898,7 +906,7 @@ autoar_extractor_do_sanitize_pathname (AutoarExtractor *self,
     g_file_has_prefix (extracted_filename, destination);
 
   if (!valid_filename) {
-    g_autofree char *basename;
+    g_autofree char *basename = NULL;
 
     basename = g_file_get_basename (extracted_filename);
 
@@ -909,7 +917,7 @@ autoar_extractor_do_sanitize_pathname (AutoarExtractor *self,
 
   if (self->prefix != NULL && self->new_prefix != NULL &&
       !g_file_equal (self->prefix, self->new_prefix)) {
-    g_autofree char *relative_path;
+    g_autofree char *relative_path = NULL;
     /* Replace the old prefix with the new one */
     relative_path = g_file_get_relative_path (self->prefix,
                                               extracted_filename);
@@ -964,6 +972,7 @@ autoar_extractor_check_file_conflict (AutoarExtractor *self,
 
   /* Check also parents for conflict to be sure it is directory. */
   parent = g_file_get_parent (file);
+  g_return_val_if_fail (parent, NULL);
   return autoar_extractor_check_file_conflict (self, parent, AE_IFDIR);
 }
 
@@ -983,7 +992,7 @@ autoar_extractor_do_write_entry (AutoarExtractor      *self,
   {
     GFile *parent;
     parent = g_file_get_parent (dest);
-    if (!g_file_query_exists (parent, self->cancellable))
+    if (parent && !g_file_query_exists (parent, self->cancellable))
       g_file_make_directory_with_parents (parent,
                                           self->cancellable,
                                           NULL);
@@ -1664,11 +1673,15 @@ autoar_extractor_step_scan_toplevel (AutoarExtractor *self)
       return;
     }
     self->use_raw_format = TRUE;
+
+    g_debug ("autoar_extractor_step_scan_toplevel: using raw format");
   }
 
   while ((r = archive_read_next_header (a, &entry)) == ARCHIVE_OK) {
     const char *pathname;
     g_autofree char *utf8_pathname = NULL;
+    const char *symlink_pathname;
+    const char *hardlink_pathname;
 
     if (g_cancellable_is_cancelled (self->cancellable)) {
       archive_read_free (a);
@@ -1683,28 +1696,26 @@ autoar_extractor_step_scan_toplevel (AutoarExtractor *self)
       }
     }
 
-    if (self->use_raw_format) {
-      pathname = autoar_common_get_basename_remove_extension (g_file_peek_path (self->source_file));
-      g_debug ("autoar_extractor_step_scan_toplevel: %d: raw pathname = %s",
-               self->total_files, pathname);
-    } else {
-      const char *symlink_pathname;
-      const char *hardlink_pathname;
+    pathname = archive_entry_pathname (entry);
+    utf8_pathname = autoar_common_get_utf8_pathname (pathname);
+    symlink_pathname = archive_entry_symlink (entry);
+    hardlink_pathname = archive_entry_hardlink (entry);
 
-      pathname = archive_entry_pathname (entry);
-      utf8_pathname = autoar_common_get_utf8_pathname (pathname);
-      symlink_pathname = archive_entry_symlink (entry);
-      hardlink_pathname = archive_entry_hardlink (entry);
+    /* The raw format usually doesn't propagate file name and the generic "data"
+     * string is returned instead. Let's use source basename in that case.
+     */
+    if (self->use_raw_format && g_str_equal (pathname, "data"))
+      pathname = autoar_common_get_basename_remove_extension (self->source_basename);
+
+    g_debug ("autoar_extractor_step_scan_toplevel: %d: pathname = %s%s%s%s%s%s%s",
+             self->total_files, pathname,
+             utf8_pathname ? " utf8 pathname = " : "",
+             utf8_pathname ? utf8_pathname : "",
+             symlink_pathname ? " symlink = " : "",
+             symlink_pathname ? symlink_pathname : "",
+             hardlink_pathname ? " hardlink = " : "",
+             hardlink_pathname ? hardlink_pathname : "");
 
-      g_debug ("autoar_extractor_step_scan_toplevel: %d: pathname = %s%s%s%s%s%s%s",
-               self->total_files, pathname,
-               utf8_pathname ? " utf8 pathname = " : "",
-               utf8_pathname ? utf8_pathname : "",
-               symlink_pathname ? " symlink = " : "",
-               symlink_pathname ? symlink_pathname : "",
-               hardlink_pathname ? " hardlink = " : "",
-               hardlink_pathname ? hardlink_pathname : "");
-    }
     self->files_list =
       g_list_prepend (self->files_list,
                       autoar_extractor_do_sanitize_pathname (self,
@@ -1769,8 +1780,8 @@ autoar_extractor_step_set_destination (AutoarExtractor *self)
      * the extension). If they do, then the destination should be the output
      * directory itself.
      */
-    g_autofree char *prefix_name;
-    g_autofree char *prefix_name_no_ext;
+    g_autofree char *prefix_name = NULL;
+    g_autofree char *prefix_name_no_ext = NULL;
 
     g_debug ("autoar_extractor_step_set_destination: pathname_prefix = %s",
              g_file_peek_path (self->prefix));
@@ -1802,7 +1813,7 @@ autoar_extractor_step_decide_destination (AutoarExtractor *self)
   GList *files = NULL;
   GList *l;
   GFile *new_destination = NULL;
-  g_autofree char *destination_name;
+  g_autofree char *destination_name = NULL;
 
   for (l = self->files_list; l != NULL; l = l->next) {
     char *relative_path;
@@ -1889,6 +1900,12 @@ autoar_extractor_step_extract (AutoarExtractor *self) {
     pathname = archive_entry_pathname (entry);
     hardlink = archive_entry_hardlink (entry);
 
+    /* The raw format usually doesn't propagate file name and the generic "data"
+     * string is returned instead. Let's use source basename in that case.
+     */
+    if (self->use_raw_format && g_str_equal (pathname, "data"))
+      pathname = autoar_common_get_basename_remove_extension (self->source_basename);
+
     extracted_filename =
       autoar_extractor_do_sanitize_pathname (self, pathname);
 
